@@ -7,7 +7,6 @@ import (
 	"encoding/json"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"math"
 	"net"
 	"os"
@@ -17,6 +16,7 @@ import (
 	"strings"
 	"sync"
 	"time"
+	"golang.org/x/time/rate"
 
 	"github.com/denisbrodbeck/machineid"
 	log "github.com/schollz/logger"
@@ -73,6 +73,7 @@ type Options struct {
 	Overwrite      bool
 	Curve          string
 	HashAlgorithm  string
+	ThrottleUpload string
 }
 
 // Client holds the state of the croc transfer
@@ -84,7 +85,7 @@ type Client struct {
 
 	// steps involved in forming relationship
 	Step1ChannelSecured       bool
-	Step2FileInfoTransfered   bool
+	Step2FileInfoTransferred   bool
 	Step3RecipientRequestFile bool
 	Step4FileTransfer         bool
 	Step5CloseChannels        bool
@@ -103,8 +104,9 @@ type Client struct {
 	LastFolder             string
 
 	TotalSent             int64
-	TotalChunksTransfered int
+	TotalChunksTransferred int
 	chunkMap              map[uint64]struct{}
+	limiter               *rate.Limiter
 
 	// tcp connections
 	conn []*comm.Comm
@@ -118,7 +120,7 @@ type Client struct {
 	numfinished             int
 	quit                    chan bool
 	finishedNum             int
-	numberOfTransferedFiles int
+	numberOfTransferredFiles int
 }
 
 // Chunk contains information about the
@@ -175,6 +177,37 @@ func New(ops Options) (c *Client, err error) {
 
 	c.conn = make([]*comm.Comm, 16)
 
+	// initialize throttler
+	if len(c.Options.ThrottleUpload) > 1 && c.Options.IsSender {
+		upload := c.Options.ThrottleUpload[:len(c.Options.ThrottleUpload)-1]
+		uploadLimit, err := strconv.ParseInt(upload, 10, 64)
+		if err != nil {
+			panic("Could not parse given Upload Limit")
+		}
+		minBurstSize := models.TCP_BUFFER_SIZE
+		var rt rate.Limit
+		switch unit := string(c.Options.ThrottleUpload[len(c.Options.ThrottleUpload)-1:]); unit {
+		case "g", "G":
+			uploadLimit = uploadLimit*1024*1024*1024
+		case "m", "M":
+			uploadLimit = uploadLimit*1024*1024
+		case "k", "K":
+			uploadLimit = uploadLimit*1024
+		default:
+			uploadLimit, err = strconv.ParseInt(c.Options.ThrottleUpload, 10, 64)
+			if err != nil {
+				panic("Could not parse given Upload Limit")
+			}
+		}
+		// Somehow 4* is neccessary
+		rt = rate.Every(time.Second / (4*time.Duration(uploadLimit)))
+		if (int(uploadLimit) > minBurstSize) {
+			minBurstSize = int(uploadLimit)
+		}
+		c.limiter = rate.NewLimiter(rt, minBurstSize)
+		log.Debugf("Throttling Upload to %#v", c.limiter.Limit())
+	}
+
 	// initialize pake for recipient
 	if !c.Options.IsSender {
 		c.Pake, err = pake.InitCurve([]byte(c.Options.SharedSecret[5:]), 0, c.Options.Curve)
@@ -579,7 +612,7 @@ func (c *Client) Receive() (err error) {
 				address := net.JoinHostPort(discoveries[i].Address, portToUse)
 				errPing := tcp.PingServer(address)
 				if errPing == nil {
-					log.Debugf("succesfully pinged '%s'", address)
+					log.Debugf("successfully pinged '%s'", address)
 					c.Options.RelayAddress = address
 					c.ExternalIPConnected = c.Options.RelayAddress
 					c.Options.RelayAddress6 = ""
@@ -695,7 +728,7 @@ func (c *Client) Receive() (err error) {
 	fmt.Fprintf(os.Stderr, "\rsecuring channel...")
 	err = c.transfer(TransferOptions{})
 	if err == nil {
-		if c.numberOfTransferedFiles == 0 {
+		if c.numberOfTransferredFiles == 0 {
 			fmt.Fprintf(os.Stderr, "\rNo files transferred.")
 		}
 	}
@@ -845,7 +878,7 @@ func (c *Client) processMessageFileInfo(m message.Message) (done bool, err error
 	fmt.Fprintf(os.Stderr, "\nReceiving (<-%s)\n", c.ExternalIPConnected)
 
 	log.Debug(c.FilesToTransfer)
-	c.Step2FileInfoTransfered = true
+	c.Step2FileInfoTransferred = true
 	return
 }
 
@@ -1061,7 +1094,7 @@ func (c *Client) processMessage(payload []byte) (done bool, err error) {
 }
 
 func (c *Client) updateIfSenderChannelSecured() (err error) {
-	if c.Options.IsSender && c.Step1ChannelSecured && !c.Step2FileInfoTransfered {
+	if c.Options.IsSender && c.Step1ChannelSecured && !c.Step2FileInfoTransferred {
 		var b []byte
 		machID, _ := machineid.ID()
 		b, err = json.Marshal(SenderInfo{
@@ -1084,7 +1117,7 @@ func (c *Client) updateIfSenderChannelSecured() (err error) {
 			return
 		}
 
-		c.Step2FileInfoTransfered = true
+		c.Step2FileInfoTransferred = true
 	}
 	return
 }
@@ -1245,7 +1278,7 @@ func (c *Client) createEmptyFileAndFinish(fileInfo FileInfo, i int) (err error)
 }
 
 func (c *Client) updateIfRecipientHasFileInfo() (err error) {
-	if !(!c.Options.IsSender && c.Step2FileInfoTransfered && !c.Step3RecipientRequestFile) {
+	if !(!c.Options.IsSender && c.Step2FileInfoTransferred && !c.Step3RecipientRequestFile) {
 		return
 	}
 	// find the next file to transfer and send that number
@@ -1272,7 +1305,7 @@ func (c *Client) updateIfRecipientHasFileInfo() (err error) {
 			if err != nil {
 				return
 			} else {
-				c.numberOfTransferedFiles++
+				c.numberOfTransferredFiles++
 			}
 			continue
 		}
@@ -1310,7 +1343,7 @@ func (c *Client) updateIfRecipientHasFileInfo() (err error) {
 		if errHash != nil || !bytes.Equal(fileHash, fileInfo.Hash) {
 			finished = false
 			c.FilesToTransferCurrentNum = i
-			c.numberOfTransferedFiles++
+			c.numberOfTransferredFiles++
 			newFolder, _ := filepath.Split(fileInfo.FolderRemote)
 			if newFolder != c.LastFolder && len(c.FilesToTransfer) > 0 {
 				fmt.Fprintf(os.Stderr, "\r%s\n", newFolder)
@@ -1468,10 +1501,10 @@ func (c *Client) receiveData(i int) {
 		}
 		c.bar.Add(len(data[8:]))
 		c.TotalSent += int64(len(data[8:]))
-		c.TotalChunksTransfered++
-		// log.Debug(len(c.CurrentFileChunks), c.TotalChunksTransfered, c.TotalSent, c.FilesToTransfer[c.FilesToTransferCurrentNum].Size)
+		c.TotalChunksTransferred++
+		// log.Debug(len(c.CurrentFileChunks), c.TotalChunksTransferred, c.TotalSent, c.FilesToTransfer[c.FilesToTransferCurrentNum].Size)
 
-		if !c.CurrentFileIsClosed && (c.TotalChunksTransfered == len(c.CurrentFileChunks) || c.TotalSent == c.FilesToTransfer[c.FilesToTransferCurrentNum].Size) {
+		if !c.CurrentFileIsClosed && (c.TotalChunksTransferred == len(c.CurrentFileChunks) || c.TotalSent == c.FilesToTransfer[c.FilesToTransferCurrentNum].Size) {
 			c.CurrentFileIsClosed = true
 			log.Debug("finished receiving!")
 			if err := c.CurrentFile.Close(); err != nil {
@@ -1482,7 +1515,7 @@ func (c *Client) receiveData(i int) {
 					c.FilesToTransfer[c.FilesToTransferCurrentNum].FolderRemote,
 					c.FilesToTransfer[c.FilesToTransferCurrentNum].Name,
 				)
-				b, _ := ioutil.ReadFile(pathToFile)
+				b, _ := os.ReadFile(pathToFile)
 				fmt.Print(string(b))
 			}
 			log.Debug("sending close-sender")
@@ -1519,6 +1552,11 @@ func (c *Client) sendData(i int) {
 		n, errRead := c.fread.ReadAt(data, readingPos)
 		// log.Debugf("%d read %d bytes", i, n)
 		readingPos += int64(n)
+		if (c.limiter != nil) {
+			r := c.limiter.ReserveN(time.Now(), n)
+			log.Debugf("Limiting Upload for %d", r.Delay())
+			time.Sleep(r.Delay())
+		}
 
 		if math.Mod(curi, float64(len(c.Options.RelayPorts))) == float64(i) {
 			// check to see if this is a chunk that the recipient wants
